global regcount tdrtable() EGreport LItable()
set LogFlag 1
set EGreport 0

# Register and RAM table: Name : Offset address : Nbits : Nlocs 
set tdrtable(0)		{"Gate width        "   0x0000 32 32 }
set tdrtable(1)		{"Gate mode A       "   0x0400 8 8   }
set tdrtable(2)		{"Group A Enable    "   0x0420 8 1   }
set tdrtable(3)		{"Group A Select    "   0x0424 4 1   }
set tdrtable(4)		{"Gate mode B       "	0x0500 8 8   }
set tdrtable(5)		{"Group B Enable    "   0x0520 8 1   }
set tdrtable(6)		{"Group B Select    "   0x0524 4 1   }
set tdrtable(7)		{"Gate mode C       "	0x0600 8 8   }
set tdrtable(8)		{"Group C Enable    "   0x0620 8 1   }
set tdrtable(9)		{"Group C Select    "   0x0624 4 1   }
set tdrtable(10)	{"Gate mode D       "	0x0700 8 8   }
set tdrtable(11)	{"Group D Enable    "   0x0720 8 1   }
set tdrtable(12)	{"Group D Select    "   0x0724 4 1   }
set tdrtable(13)	{"Veto LUT          "	0x0800 32 256}
set tdrtable(14)	{"Scaler Mux        "	0x1020 5 8   }
set tdrtable(15)	{"Scaler enable     "   0x1040 8 1   }
set tdrtable(16)	{"Readout Control   "	0x2800 16 1  }
set tdrtable(17)	{"Readout Ident     "	0x3000 32 128}
set tdrtable(18)	{"Card Control      "	0x3800 16 1  }
set tdrtable(19)	{"Spare             "   0x8000 32 1  }
set tdrtable(20)	{"Timestamp re-load "   0x7000 32 1  }
set tdrtable(21)	{"Readout Enable    "   0x5000 4 8  }
set tdrtable(22)	{"AI select         "   0x6000 8 4  }

set regcount 23

# Logic Inspection register test  : 
# Name  : chip number : nbits Signal : nbits Channel :signal mask : channel mask

set LItable(0)		{"Gate, Veto	" 2 3 6 7 63}
set LItable(1)		{"ADC Readout	" 3 3 6 7 63}
set LItable(2)		{"ADC32  	" 1 5 6 31 63}
set LItable(3)		{"VXI, SHARC	" 4 8 8 255 127}

# test the TDR32 LI registers 
proc LItdr32 {} {
global LItable LogFlag EGreport	

	set LIsignal	0x4000
	set LIchannel	0x4010
	set LIchip	0x4020
	set LogFlag 0 
	set EGreport -1

# set the chip pointer register then write an id value into every location 
	set i 0
	for { set j 0 } {$j < 4 } { incr j} {
	    wrl [expr $LIchip + $j * 4 ] [ lindex $LItable($i) 1]
	    wrl [expr $LIsignal + $j * 4 ] [expr $j   ]
	    wrl [expr $LIchannel + $j * 4 ] [expr 127 - $j  ]
	}
	log " Identifier written to all LI registers "
	update idletasks
# read and check each value for each chip
	for {set i 0 } { $i < 4 } { incr i } {
	    for { set j 0 } {$j < 4 } { incr j} {
	    	wrl [expr $LIchip + $j * 4 ] [ lindex $LItable($i) 1]
		set a [expr $j & [ lindex $LItable($i) 4] ]
		set b [rdl [expr $LIsignal + $j * 4 ] ]
		set b [expr $b & [ lindex $LItable($i) 4] ]
		if { $a != $b } {
 		  log "[ lindex $LItable($i) 0 ] Signal $j:  [format "0x%02x" $a] [format "0x%02x" $b] "
		  update idletasks
		}
		set a [expr 127 - $j  ]
		set a [expr $a & [ lindex $LItable($i) 5] ]
		set b [rdl [expr $LIchannel + $j * 4 ] ]
		set b [expr $b & [ lindex $LItable($i) 5] ]
		if { $a != $b } {
 		  log "[ lindex $LItable($i) 0 ] Channel $j:  [format "0x%02x" $a] [format "0x%02x" $b] "
		  update idletasks
		}
	    }
	}
	log " Identifier read from each LI register  and checked"
	update idletasks
}
# A simple read at each of the base addresses
proc test1 {} {
global regcount tdrtable
	for {set i 0} {$i < $regcount} {incr i} {
	set reg_address [ lindex $tdrtable($i) 1 ]
	rdl $reg_address
	}
}
# Addressing test
proc test2 {} {
global regcount tdrtable LogFlag EGreport
	set LogFlag 0 
	set EGreport -1
	for {set i 0} {$i < $regcount} {incr i} {
	   set reg_address [ lindex $tdrtable($i) 1 ]
	   set nlocs [lindex $tdrtable($i) 3 ] 
	   for { set j 0} { $j < $nlocs } {incr j } {
		wrl $reg_address $j
		set reg_address [ expr  $reg_address +  4  ]
	   }
	}
	log { All writes done for Identity test }
	update idletasks
	for {set i 0} {$i < $regcount} {incr i} {
	   set reg_address [ lindex $tdrtable($i) 1 ]
	   set nlocs [ lindex $tdrtable($i) 3 ] 
	   set nbits [ lindex $tdrtable($i) 2 ] 
	   for { set j 0} { $j <  $nlocs } {incr j } {
		set b [rdl $reg_address]
		set mask [ expr 1 << $nbits ]
		set mask [ expr $mask -1 ]
		set b [ expr $b & $mask ]
		set a [ expr $j & $mask ]
		if { $b != $a } {
		  log "[ lindex $tdrtable($i) 0 ] [format "0x%04x" $reg_address] [format "0x%08x" $a] [format "0x%08x" $b] "
		  update idletasks
		}
		set reg_address [ expr  $reg_address + 4]
	   }
	}
	log { All Reads done for Identity test }
	update idletasks
}
# Running ones, and noughts test
proc test3 {} {
global regcount tdrtable LogFlag EGreport
	set LogFlag 0 
	set EGreport -1
	for { set i 0} { $i < $regcount } { incr i } {
	   log "[lindex $tdrtable($i) 0]"
	   update idletasks
	   set reg_address [ lindex $tdrtable($i) 1 ]
	   set nlocs [ lindex $tdrtable($i) 3 ] 
	   set nbits [ lindex $tdrtable($i) 2 ] 
	   set mask  [ expr 1 << $nbits ]
	   set mask  [ expr $mask -1 ]
	   for { set j 0 } { $j < $nlocs } { incr j } {
		set l 1
		for { set k 0 } { $k < $nbits } { incr k } {
		    wrl $reg_address $l
		    set b [rdl $reg_address]
		    set b [ expr $b & $mask ]
		    set a [ expr $l & $mask ]
		    if { $b != $a } {
		      log "[ lindex $tdrtable($i) 0 ] [format "0x%04x" $reg_address] [format "0x%08x" $a] [format "0x%08x" $b] "
		      update idletasks
		    }
		    set l [expr $l << 1 ]
		}
		wrl $reg_address 0 ;# to catch the board reset bits
		set reg_address [ expr  $reg_address + 4]
	   }
	   set reg_address [ lindex $tdrtable($i) 1 ]
	   for { set j 0 } { $j < $nlocs } { incr j } {
		set l -2
		for { set k 0 } { $k < $nbits } { incr k } {
		    wrl $reg_address $l
		    set b [rdl $reg_address]
		    set b [ expr $b & $mask ]
		    set a [ expr $l & $mask ]
		    if { $b != $a } {
		      log "[ lindex $tdrtable($i) 0 ] [format "0x%04x" $reg_address] [format "0x%08x" $a] [format "0x%08x" $b] "
		      update idletasks
		    }
		    set l [expr ( $l << 1 ) + 1]
		}
		wrl $reg_address 0 ;# to catch the board reset bits
		set reg_address [ expr  $reg_address + 4]
	   }
	}
}

# Running the tests
set-busy
test1
test2
test3
LItdr32
set LogFlag 1
set EGreport 0
clear-busy
log {That's all Folks !}
update idletasks
