# silena761f.tcl---Silena 761F amplifier control frame


set frame AMPS
set frametitle "Silena 761F Amplifier Setup & Monitor"
set icontitle "Amps"

set started 0

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource VME}
    
set savedirectory Silena761fAmpSetup

set MAXmodule 16

set modulenum 1
set ampnum 0
set allmodules 0
set allamps 0
set ampsel 1
set modulesel 1
set ampselection {1}
set moduleselection {1}

set stime 0
set cgain 0
set inversion 0
set fgain 0
set polezero 0
set offset 0

set stimemenu [list fastest fast slow slowest]
set cgainmenu [list 1 2 4 8 16 32 64 128]

set selectmodulemenu   {}
for {set i 1} {$i <= $MAXmodule} { incr i} {append  selectmodulemenu "$i "}
set modulelist $selectmodulemenu

set selectampmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set amplist $selectampmenu

set selectmuxmenu {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 Off}

for {set i 1} {$i <= $MAXmodule} { incr i} {set modulepresent($i) 0}

set AmpStatus ""
set Status ""

set Request ""
set Reply ""


proc start args {
   global started
   if {$started == 0} then {make-frame; EGbindings} else {open-frame}
   set started 1
   set-footer "Probing for amplifier modules"
   probe-for-modules
   clear-busy
   set-footer "Ready"
}

proc define-module {} {
  global resource
  set-footer "deleting all Silena761f Amp registers"
  EG "register :pattern '/$resource/Silena761fAmp.*' !delete"

#    write command
  define-register  CMD        "-p rw -z x"
}

proc define-register {name attributes} {
   global resource
   set register /$resource/Silena761fAmp.$name
   set-footer "defining register $register"
   catch {EG "register :name '$register' :class 'xSilena761FAmp' !new"}
   if {$attributes != ""} then {EG "register :name '$register' :object '$attributes'"}
   clear-footer
}

proc write-register {name value} {
   global resource
   set register /$resource/Silena761fAmp.$name
   EG "register :name '$register' :value '$value'"
}

proc read-register {name} {
   global resource
   set register /$resource/Silena761fAmp.$name
   set value [EG "register :name '$register' ?value"]
   return $value
}

proc 761F_message {module task Opcode args} {
    global Request Reply
    global EGallow

    switch $Opcode {

        UNIT_GETINFO     {set opcode 0x07; set ack 0x09}
        UNIT_GETALM      {set opcode 0x01; set ack 0x0a}
        UNIT_SETMUX      {set opcode 0x14; set ack 0x15}
        UNIT_GETMUX      {set opcode 0x16; set ack 0x17}
        UNIT_SAVEPAR     {set opcode 0x10; set ack 0x11}
        UNIT_LOADPAR     {set opcode 0x12; set ack 0x13}
        TASK_GETPAR      {set opcode 0x011d; set ack 0x1e}
        TASK_SETPAR      {set opcode 0x011f; set ack 0x20}

        default          {midas-warning "Unknown message request $Opcode"; return ""}
    }

    set opcodeH [expr $opcode >> 8]
    set opcodeL [expr $opcode & 255]
    set UserDataLength [expr [llength $args] + 2]
    set checksum [expr $module +$task + $opcodeH +$opcodeL + $UserDataLength]
    foreach i $args {incr checksum  $i}
    set checksumH [expr $checksum >> 8]
    set checksumL [expr $checksum & 255]
    set Request "$module 0xAA $module $task 0 0 $opcodeL $opcodeH $UserDataLength 0 $args $checksumL $checksumH 0xCC"

    if {$Opcode == "UNIT_SAVEPAR" || $Opcode == "UNIT_LOADPAR"} then {set EGallow 0x2000a}

    write-register CMD $Request
    set Reply [read-register CMD]

    if {$Opcode == "UNIT_SAVEPAR" || $Opcode == "UNIT_LOADPAR"} then {set EGallow -1;after 250; return}

    if {$EGallow == 0x2000a && [llength $Reply] == 0} then {return $Reply}
    if {[llength $Reply] == 0} then {
         midas-warning "Module $module: Not responding"
         return $Reply
    }
    set opcode [lindex $Reply 5]

    if  {$opcode != $ack} then {
        if {$opcode == 0x00} then {midas-warning "Module $module: Request rejected - bad checksum"; return ""}
        if {$opcode == 0x0b} then {midas-warning "Module $module: Request rejected - code [lindex $Reply 11]"; return ""}
        midas-warning "Bad response $Reply"
        return ""
    }
    return [lrange $Reply 9 [expr [llength $Reply] - 4]]
}

proc get_parameters {m a} {
   global AmpStatus
   global stime cgain inversion fgain polezero offset

   set AmpStatus [761F_message $m $a TASK_GETPAR]

   if {$AmpStatus == "" || [llength $AmpStatus] != 4} {
       set r [midas-confirmation "Bad return from Silena761f module=$m amp=$a.\n\
          Do you wish to reset the parameters for this amp which may correct the problem?" \
          "Yes" "No"]
       if {$r == 0} then {
           set inversion 1; set cgain 0; set stime 0; set fgain 0; set polezero 0; set offset 0
           set_parameters $m $a
           set AmpStatus [761F_message $m $a TASK_GETPAR]
           if {$AmpStatus == "" || [llength $AmpStatus] != 4} {
                midas-warning "Silena761f module=$m amp=$a is still faulty."
                set AmpStatus [list 0 0 0 0]
           }
       } else {
           set AmpStatus [list 0 0 0 0]
       }
   }

   set stime [expr [first $AmpStatus] & 3]
   set cgain [expr ([first $AmpStatus] >> 2) & 7]
   set inversion [expr ([first $AmpStatus] >> 5) & 3]
   set fgain [second $AmpStatus]
   set polezero [third $AmpStatus]
   set offset [fourth $AmpStatus]
}

proc set_parameters {m a} {
   global stime cgain inversion fgain polezero offset

   set port [expr ((($inversion << 3) + $cgain) << 2) + $stime]
   761F_message $m $a TASK_SETPAR $port $fgain $polezero $offset
}

proc do_set {p v} {
   global EGreport
   global allmodules allamps modulepresent
   global modulenum selectmodulemenu moduleselection
   global ampnum selectampmenu ampselection
   global stime cgain inversion fgain polezero offset
   if {$allmodules == 0} then {set modulelist [list $modulenum]}
   if {$allmodules == 1} then {set modulelist $selectmodulemenu}
   if {$allmodules == -1} then {set modulelist $moduleselection}
   if {$allamps == 0} then {set amplist [list $ampnum]}
   if {$allamps == 1} then {set amplist $selectampmenu}
   if {$allamps == -1} then {set amplist $ampselection}

   set-footer "Busy"
   set-busy
   set EGreport_SAVE $EGreport
   set EGreport -1

   foreach module $modulelist {
      if {$modulepresent($module) != 0} then {
         foreach amp $amplist {
            get_parameters $module $amp
            set $p $v
            set_parameters $module $amp
         }
      }
   }
   do-update

   set EGreport $EGreport_SAVE
   clear-busy
   set-footer "Ready"
}

proc do_cmd {p} {
   global allmodules allamps modulepresent
   global modulenum selectmodulemenu moduleselection
   global ampnum selectampmenu ampselection
   global stime cgain inversion fgain polezero offset
   if {$allmodules == 0} then {set modulelist [list $modulenum]}
   if {$allmodules == 1} then {set modulelist $selectmodulemenu}
   if {$allmodules == -1} then {set modulelist $moduleselection}
   if {$allamps == 0} then {set amplist [list $ampnum]}
   if {$allamps == 1} then {set amplist $selectampmenu}
   if {$allamps == -1} then {set amplist $ampselection}

   foreach module $modulelist {
      if {$modulepresent($module) != 0} then {
         761F_message $module 255 UNIT_$p
      }
   }
   do-update
}

proc do_mux {v} {
   global modulenum
   global allmodules allamps modulepresent
   global modulenum selectmodulemenu moduleselection
   global MPXPresent
#PMJ
    global ampnum

   if {$allmodules == 0} then {set modulelist [list $modulenum]}
   if {$allmodules == 1} then {set modulelist $selectmodulemenu}
   if {$allmodules == -1} then {set modulelist $moduleselection}

   set amp $v
   if {$amp == 16} then {set amp 255}

#PMJ
   if {$amp < 16} then {set ampnum $amp}

   if {$MPXPresent == 0} {

       foreach module $modulelist {
          if {$modulepresent($module) != 0} then {
             761F_message $module $amp UNIT_SETMUX $amp
          }
       }
   } else {
   
       foreach module $selectmodulemenu {
          if {$modulepresent($module) != 0} then {
             761F_message $module 255 UNIT_SETMUX 255
          }
       }
       761F_message $modulenum $amp UNIT_SETMUX $amp
   }

   connect-MPX

   do-update
}

proc connect-MPX {} {}

proc module v {
   global modulenum modulepresent
   global ampnum
   if {$modulepresent($v) == 0} then {
      midas-warning "Module $v may not be connected - Use Probe command if you think it is connected to check for existance"
      repaint
      return
   }
   set modulenum $v
#   761F_message $modulenum $ampnum UNIT_SETMUX $ampnum
   do-update
}

proc amplifier v {
   global modulenum ampnum
   set ampnum $v
#   761F_message $modulenum $ampnum UNIT_SETMUX $ampnum
   do-update
}

proc probe-for-modules {} {
#    probe for modules to check presence
   global EGallow
   global selectmodulemenu modulepresent modulenum ampnum

   define-module

   set modulenum 0
   set ampnum 0
   set-busy
   set pattern 0
   :modify MODULESINUSE :value $pattern
   foreach module $selectmodulemenu {
      set-footer "Probing for module $module"
      set EGallow 0x2000a
      set reply [761F_message $module 0 UNIT_GETINFO]
      set EGallow -1
      if {[string length $reply] > 0} then {
#    response (of some sort) from module
          set modulepresent($module) 1
          if {$modulenum == 0} then {module $module}
          incr pattern [expr 1 << [expr $module - 1]]
      } else {
          set modulepresent($module) 0
      }
   :modify MODULESINUSE :value $pattern
   }
   clear-busy
   clear-footer
   if {$modulenum == 0} then {
      midas-warning "No amplifier modules found!"
   } else {
      do-update
   }
}

proc do-update {} {
   global modulenum ampnum
   global stime cgain inversion fgain polezero offset

   get_parameters $modulenum $ampnum

#    update display

   :modify STIME :value $stime
   :modify CGAIN :value $cgain
   :modify INVERSION :value [expr $inversion - 1]
   :modify FGAIN :value $fgain
   :modify POLEZERO :value $polezero
   :modify OFFSET :value $offset
#   and the MUX setting
   set AmpStatus [761F_message $modulenum 0 UNIT_GETMUX]
   set vl [lindex $AmpStatus 0]
   if {$vl == 0xff} then {set vl 16}
   :modify SELECT :value $vl
}

proc show-status {} {
   global env AmpStatus
   global selectmodulemenu modulenum selectampmenu stimemenu cgainmenu
   global modulepresent allmodules moduleselection
   global stime cgain inversion fgain polezero offset

   if {$allmodules == 0} then {set modulelist [list $modulenum]}
   if {$allmodules == 1} then {set modulelist $selectmodulemenu}
   if {$allmodules == -1} then {set modulelist $moduleselection}

   set contents ""
   set-busy
   foreach module $modulelist {
      if {$modulepresent($module) != 0} then {
         set-footer "Getting amplifier settings from module $module..."
         append contents "Module $module\nAmplifier #  --  time constant;  coarse gain;  fine gain;  pole zero;  offset DAC; inversion\n"
         foreach amp $selectampmenu {
            get_parameters $module $amp
            if {"$AmpStatus" == ""} then {break}
            append contents "Amp $amp  --   $stime     $cgain     $fgain     $polezero     $offset     $inversion \n"
         }
      }
   }
   midas-information $contents
   clear-busy
   clear-footer
}

proc repaint {} {
   global allmodules allamps
   global modulenum ampnum
  :modify MODULE-NUM :value $modulenum
  if {$allmodules == -1} then {
     :modify ALL-MODULES :value 2
  } else {
     :modify ALL-MODULES :value $allmodules
  }
  :modify AMP-NUM   :value $ampnum
  if {$allamps == -1} then {
     :modify ALL-AMPS  :value 2
  } else {
     :modify ALL-AMPs :value $allamps
  }
}

proc click-save {w} {
   global env EGreport AmpStatus Status savedirectory
   global selectmodulemenu selectampmenu modulenum
   global modulepresent

   catch {file delete [file join $env(EXPERIMENTDIR) $savedirectory]}
   if {![file exists [file join $env(EXPERIMENTDIR) $savedirectory]]} then {
      set z [catch {:file mkdir [file join $env(EXPERIMENTDIR) $savedirectory]} m]
      if {$z != 0} then {midas-warning "Cannot create directory [file join $env(EXPERIMENTDIR) $savedirectory]\n $m"}
   }

   if {$w == "ALL"} then {
      set modulelist $selectmodulemenu
   } else {
      set modulelist $modulenum
   }
   set amplist $selectampmenu

   set-busy
   set EGreport_SAVE $EGreport
   set EGreport -1

   set contents ""
   foreach module $modulelist {
      set Status {}
      if {$modulepresent($module) != 0} then {
         set-footer "Saving amplifier settings from module $module..."
         append contents "Amplifier settings saved from module $module amplifiers"
         foreach amp $amplist {
#    obtain values for this amplifier
            get_parameters $module $amp
            if {"$AmpStatus" != ""} then {
               lappend Status $AmpStatus
               append contents " $amp"
            } else {
               break
            }
         }
#    save amplifier status to file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory c$module] w]
         puts $f $Status
         close $f
         append contents " to file [file join $env(EXPERIMENTDIR) $savedirectory c$module]\n"
      }
   }

   set EGreport $EGreport_SAVE
   clear-busy

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been saved!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings saved to $env(EXPERIMENTDIR)/$savedirectory"

   do-update
}

proc click-restore {w} {
     global env EGreport
     global AmpStatus Status savedirectory
     global modulenum selectmodulemenu modulepresent hexchars
     global stime cgain inversion fgain polezero offset

     if {$w == "ALL"} then {
        set modulelist $selectmodulemenu
     } else {
        set modulelist $modulenum
     }

     set-busy
     set EGreport_SAVE $EGreport
     set EGreport -1
     set contents ""

     foreach module $modulelist {
       if {$modulepresent($module) != 0} then {
         set-footer "Restoring Amplifier settings to module $module..."
         append contents "Amplifier settings restored to module $module amplifiers"

         set f [open [file join $env(EXPERIMENTDIR) $savedirectory c$module] r]
         set Status [gets $f]
         close $f

         set amp 0
         foreach AmpStatus $Status {

             set n [llength $AmpStatus]

             if {$n >= 1} {set stime [expr [first $AmpStatus] & 3]} else {set stime 0}
             if {$n >= 1} {set cgain [expr ([first $AmpStatus] >> 2) & 7]} else {set cgain 0}
             if {$n >= 1} {set inversion [expr ([first $AmpStatus] >> 5) & 3]} else {set inversion 0}
             if {$n >= 2} {set fgain [second $AmpStatus]} else {set fgain 0}
             if {$n >= 3} {set polezero [third $AmpStatus]} else {set polezero 0}
             if {$n >= 4} {set offset [fourth $AmpStatus]} else {set offset 0}

             set_parameters $module $amp
             incr amp
         }
         append contents " from file [file join $env(EXPERIMENTDIR) $savedirectory c$module]\n"
       }
     }

     set EGreport $EGreport_SAVE
     clear-busy

     if {$contents == ""} then {
       midas-warning "Warning - Nothing has been restored!"
     } else {
       midas-report "$contents"
     }

     set-footer "Amplifier settings restored from $env(EXPERIMENTDIR)/$savedirectory"

     do-update
}

proc click-redisplay {} {repaint; do-update}

proc click arg {
   global debug
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      MODULE-SEL    -
      MODULE-NUM    {module $v}
      MODULESEL     {click-moduleselection $v}

      ALL-MODULES   {click-whichmodules $v}

      AMP-SEL      -
      AMP-NUM      {amplifier $v}
      AMPSEL       {click-ampselection $v}

      ALL-AMPS     {click-whichamps $v}

      SAVEPAR      -
      LOADPAR      {do_cmd $w}

      STIME        -
      CGAIN        -
      POLEZERO     -
      OFFSET       -
      FGAIN        {do_set [string tolower $w] $v}
      INVERSION    {do_set [string tolower $w] [expr $v + 1]}

      SELECT       {do_mux $v}

      PROBE        {probe-for-modules}
      RESET        {reset-interface}
      STATUS       {show-status}

      SAVE         {click-save CURRENT}
      SAVEALL      {click-save ALL}
      RESTORE      {click-restore CURRENT}
      RESTOREALL   {click-restore ALL}

      REDISPLAY    {click-redisplay}
      default      {puts stdout $arg}
   }
   repaint
}


proc make-frame {} {
  global env
  global selectmodulemenu selectampmenu stimemenu cgainmenu selectmuxmenu
  global MAXmodule

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $MAXmodule :w 5  :valuex 16c
      :menubutton MODULE-SEL :label "select:" :strings $selectmodulemenu
      :hskip 10
      :choice ALL-MODULES :label "Act on modules?" :valuex 18c :strings [list Current All Select] :value 0
      :next-row T
      :number AMP-NUM     :label "Amplifier number:" :min 0 :max 15 :w 5 :valuex 16c
      :menubutton AMP-SEL :label "select:" :strings $selectampmenu
      :hskip 10
      :choice ALL-AMPS :label "Act on amplifiers?" :valuex 18c :strings [list Current All Select] :value 0
      :next-row T
      :choice MODULESINUSE :label "In Use" :valuex 6c :exclusive F :pad 0 :strings $selectmodulemenu :w 3 :deaf T
      :next-row T
      :choice SELECT       :label "MUX"    :valuex 6c :exclusive T :strings $selectmuxmenu :w 3 

    :panel P1 :layout L :padx 10
      :layout H; :col-gap 15; :row-gap 15
      
      :hskip 40
      :choice-stack STIME  :label "Shaping Time:" :exclusive T  :strings $stimemenu :value 0 
      :choice INVERSION    :label "Inversion:"    :exclusive T  :strings [list Off On] :value 0
      :choice-stack CGAIN  :label "Coarse Gain:"  :exclusive T  :strings $cgainmenu :value 0
      :next-row T 
      :slider POLEZERO     :label "Pole Zero" :min 0 :max 255 :valuex 30c
      :next-row T
      :slider OFFSET       :label "Offset"    :min 0 :max 255 :valuex 30c
      :next-row T
      :slider FGAIN        :label "Fine Gain" :min 0 :max 255 :valuex 30c

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :col-gap 10; :row-gap 5
      :button PROBE    :label "Probe for modules"
      :button STATUS   :label "Show Settings"
      :button SAVEPAR  :label "Save to NVRAM"
      :button LOADPAR  :label "Load from NVRAM"
      :next-row T
      :col-gap 0
      :button SAVEALL     :label "Save All Modules"
      :button SAVE        :label "Save Current Module"
      :button RESTOREALL  :label "Restore All Modules"
      :button RESTORE     :label "Restore Current Module"
      :next-row T
      :button REDISPLAY
      :hskip 100
      :button HELP  :command click-help

      :show T

}

proc make-moduleSELframe {} {
  global modulesel modulelist

   :frame MODULESEL-FRAME :label "Amplifier Module Selection"  :show-footer F :resize F :transient T
   :panel MODULESEL-PANEL :layout T
   :choice MODULESEL :exclusive F :strings $modulelist :value $modulesel
   :show T
}

proc make-ampSELframe {} {
  global ampsel amplist

   :frame AMPSEL-FRAME :label "Amplifier Amp Selection"  :show-footer F :resize F :transient T
   :panel AMPSEL-PANEL :layout T
   :choice AMPSEL :exclusive F :strings $amplist :value $ampsel
   :show T
}

proc click-whichamps v {
  global allamps
  switch $v {
    0  -
    1  {set allamps $v}
    2  {
       set allamps -1
       make-ampSELframe
    }
  }
}

proc click-whichmodules v {
  global allmodules
  switch $v {
    0  -
    1  {set allmodules $v}
    2  {
       set allmodules -1
       make-moduleSELframe
    }
  }
}

proc click-ampselection v {
  global ampsel amplist ampselection
  set ampsel $v
  set ampselection {}
  foreach i [list-take $amplist [bits-in $ampsel]] {
    lappend  ampselection $i
  }
}

proc click-moduleselection v {
  global modulesel modulelist moduleselection
  set modulesel $v
  set moduleselection {}
  foreach i [list-take $modulelist [bits-in $modulesel]] {
    lappend moduleselection  $i
  }
}
