# V551.tcl---CAEN V551 C-RAMS Sequencer module control frame

set frame "V551"
set frametitle "CAEN V551 C-RAMS Sequencer"
set icontitle "V551"

set modulename v551
set moduletype  "CAEN V551"

loadsource common.vme

set allmodules 0


for {set i 1} {$i <= 5} {incr i} {set w2r(T$i)  T$i}
set w2r(NUMCHAN) NumChan
set w2r(ID)     Status.IntDelay
set w2r(V)      Status.Veto
set w2r(AT)     Status.AutoTrig
set w2r(DY)     Status.DataRdy
set w2r(B)      Status.Busy
set w2r(AS)     Status.ActSeq
set w2r(TM)     Test.TM
set w2r(CL)     Test.CL
set w2r(SL)     Test.SL
set w2r(PL)     Test.PL
set w2r(CLEAR)   Clear
set w2r(TRIGGER) Trigger
set w2r(STATUS)  Status
set w2r(TEST)    Test


proc click-widget {w value} {
    global w2r
    global modulename module
    global allmodules

    set register $w2r($w)

    if {$allmodules == 0} then {
        write-vme-register $modulename#$module.$register $value
    } else {
        write-vme-registers $modulename#\[?,??\].$register $value
    }

    click-redisplay
}

proc update-widget {w} {
    global w2r modulename module
    
    set register $w2r($w)
    set value [read-vme-register $modulename#$module.$register]
    :modify $w :value $value
}

proc click-select-module {v} {
   global module maxmodule
   set module $v
   if {$module<1} then {set module 1}
   if {$module>$maxmodule} then {set module $maxmodule}
   click-redisplay
}

proc click-all-modules {v} {global allmodules; set allmodules $v}

proc click-ID {} {
    global modulename module
    foreach id {1 2 3} {
       set vl [read-vme-register $modulename#$module.ModID#$id]
       insert-log "V551 Module Identifier Word $id = $vl"
    }
}
   
proc redisplay-module {} {
   global w2r modulename module maxmodule

   set maxmodule [inform vmecontrol inquire [set modulename]count]
   set menu {}
   for {set i 1} {$i <= $maxmodule} {incr i} {lappend menu $i}
   :modify MODULE-SEL :strings $menu
   :modify MODULE-NUM :max $maxmodule
   :modify MODULE-NUM :value $module

   foreach w {NUMCHAN T1 T2 T3 T4 T5 STATUS TEST} {update-widget $w}
}

proc click-redisplay {} {
   global EGreport

   clear-footer
   set EGreport -1
   set-busy

   redisplay-module

   clear-busy
   set EGreport 0
}

proc click-registers {} {
   global EGreport moduletype modulename module registers_table

   if {[info procs [set moduletype]_registers] == ""} then  \
        {set-footer "No [set moduletype]_registers procedure"; return}

   [set moduletype]_registers

   clear-footer
   set EGreport -1

   set info "[set moduletype] - module $modulename#$module\n\n"

   foreach item $registers_table {
       set description [first $item]
       set register [second $item]
       set data [read-vme-register $modulename#$module.$register]
       append info "[format "%-40s %-20s" $description $data]\n"
   }

  midas-information $info

   set EGreport 0
}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc midas-information {text} {

    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :transient T

    :galley G :deaf T :strings $text :font fixed :rows 50
    :show T
}


proc click arg {
   global debug
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM  {click-select-module $v}

      ALL-MODULES {click-all-modules $v}
      
      STATUS    {click-widget $w [expr $v & 7]}
      
      CLEAR -
      TRIGGER   {click-widget $w 0}
      
      REDISPLAY      {click-redisplay}
      ID             {click-ID}
      
      TEST -
      T*        {click-widget $w $v}
      
      ENGINEERING       {click-$v}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}


proc make-frame {} {
  global env
  global module maxmodule modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

     :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
     :menubutton MODULE-SEL :label "select:" :strings $modulelist
     :hskip 200
     :checkbox ALL-MODULES :label "Act on ALL modules?"

     :next-row T
     :text NUMCHAN :label "Number of Channels for readout"
     
     :next-row T
     :slider T1 :min 0 :max 255 :valuex 5c
     :slider T2 :min 0 :max 255 :valuex 5c
     :slider T3 :min 0 :max 255 :valuex 5c
     :next-row T
     :slider T4 :min 0 :max 255 :valuex 5c
     :slider T5 :min 0 :max 255 :valuex 5c
     
     :next-row T
     :message M1 :label "Status Register" :bold T
     :next-row T
     :choice STATUS :label "" :strings [list "Internal Delay" Veto Autotrigger "Data Ready" Busy "Active Sequence"] :w 16 :exclusive F
     
     :next-row T
     :message M2 :label "Test Register" :bold T
     :next-row T
     :choice TEST   :label "" :strings [list "Test Mode" "Clock Level" "Shift Level" "Test Pulse Level"] :w 16 :exclusive F

  
   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY
      :button CLEAR    :label "Generate CLEAR"
      :button TRIGGER  :label "Generate TRIGGER"
      :button ID       :label "Identify Module"
      :menubutton ENGINEERING :label "Engineering"  :menu [list software registers]

  :show T

}
